@@ -4,19 +4,22 @@ import asyncio
 import os
 from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
-from jinja2 import Environment
+from jinja2 import Environment, FileSystemLoader, TemplateNotFound
 from loguru import logger
+from nanoid import generate  # Added for agent ID generation
 
 from quantalogic.tools import Tool
 
 from .agent_config import AgentConfig
 from .codeact_agent import CodeActAgent
 from .constants import MAX_TOKENS
-from .conversation_history_manager import ConversationHistoryManager
+from .conversation_manager import ConversationManager
 from .executor import BaseExecutor, Executor
+from .message import Message
 from .plugin_manager import PluginManager
 from .reasoner import BaseReasoner, Reasoner
-from .tools import RetrieveStepTool
+from .templates import jinja_env as default_jinja_env
+from .tools.retrieve_message_tool import RetrieveMessageTool
 from .tools_manager import get_default_tools
 from .utils import process_tools
 
@@ -40,6 +43,7 @@ class Agent:
             logger.error(f"Failed to initialize config: {e}. Using default configuration.")
             config = AgentConfig()
 
+        self.id = generate(size=21)  # New: Unique ID for the agent
         self.config = config
         self.plugin_manager = PluginManager()
         try:
@@ -48,60 +52,138 @@ class Agent:
             logger.error(f"Failed to load plugins: {e}")
         self.model: str = config.model
         self.temperature: float = config.temperature  # Added temperature
-        self.default_tools: List[Tool] = self._get_tools()
+        self.max_iterations: int = config.max_iterations
+        self.personality = config.personality
+        self.name: Optional[str] = config.name if config.name else f"agent_{self.id[:8]}"  # Use config name or default
+        self.max_history_tokens: int = config.max_history_tokens
+        # Initialize conversation manager before getting tools to ensure it's available
+        self.conversation_manager = ConversationManager(max_tokens=self.max_history_tokens)
+        # Now get tools with conversation_manager initialized
+        self.default_tools: List[Tool] = self._load_and_configure_tools()
         # If no tools are loaded, do not fall back to all registered tools
         if not self.default_tools and (not config.enabled_toolboxes or not config.tools):
             logger.info("No tools loaded from configuration; no toolboxes specified.")
-        self.max_iterations: int = config.max_iterations
-        self.personality = config.personality
-        self.backstory = config.backstory
-        self.sop: Optional[str] = config.sop
-        self.name: Optional[str] = config.name
-        self.max_history_tokens: int = config.max_history_tokens
-        self.jinja_env: Environment = config.jinja_env
+        # Configure Jinja environment: support TemplateConfig or raw dict
+        tpl = config.template
+        td = None
+        if isinstance(tpl, dict):
+            td = tpl.get("template_dir")
+        elif hasattr(tpl, "template_dir"):
+            td = tpl.template_dir
+        if td:
+            self.jinja_env = Environment(loader=FileSystemLoader(str(td)))
+        else:
+            self.jinja_env = default_jinja_env
         self._observers: List[Tuple[Callable, List[str]]] = []
         self.last_solve_context_vars: Dict = {}
-        self.default_reasoner_name: str = config.reasoner.get("name", config.reasoner_name)
-        self.default_executor_name: str = config.executor.get("name", config.executor_name)
-        self.conversation_history_manager = ConversationHistoryManager(max_tokens=self.max_history_tokens)
+        self.default_reasoner_name: str = config.reasoner.name
+        self.default_executor_name: str = config.executor.name
         self.react_agent = CodeActAgent(
             model=self.model,
             tools=self.default_tools,
             max_iterations=self.max_iterations,
             max_history_tokens=self.max_history_tokens,
             system_prompt=self._build_system_prompt(),
-            reasoner=Reasoner(self.model, self.default_tools, temperature=self.temperature),
-            executor=Executor(self.default_tools, self._notify_observers),
-            conversation_history_manager=self.conversation_history_manager,
-            temperature=self.temperature  # Pass temperature
+            reasoner=Reasoner(
+                self.model,
+                self.default_tools,
+                temperature=self.temperature,
+                agent_id=self.id,
+                agent_name=self.name
+            ),
+            executor=Executor(
+                self.default_tools,
+                self._notify_observers,
+                self.conversation_manager,
+                agent_id=self.id,  # New: Pass agent ID
+                agent_name=self.name  # New: Pass agent name
+            ),
+            conversation_manager=self.conversation_manager,
+            temperature=self.temperature,
+            agent_id=self.id,  # New: Pass agent ID
+            agent_name=self.name  # New: Pass agent name
         )
 
-    def _get_tools(self) -> List[Tool]:
-        """Load tools, applying tools_config if provided."""
+    @property
+    def tools(self) -> List[Tool]:
+        """Get the configured tools based on current toolbox settings.
+        
+        This property provides access to the agent's tools, loading and configuring them
+        based on the current toolbox settings and configurations.
+        
+        Returns:
+            List[Tool]: The list of configured tools
+        """
+        return self._load_and_configure_tools()
+    
+    def refresh_tools(self) -> None:
+        """Explicitly refresh the tools after configuration changes.
+        
+        This method should be called when toolbox configurations change and
+        the tools need to be reloaded with the new settings.
+        """
+        self.default_tools = self._load_and_configure_tools()
+    
+    def _load_and_configure_tools(self) -> List[Tool]:
+        """Load and configure tools based on current settings.
+        
+        This is an internal method that handles the loading and configuration of tools
+        based on the current toolbox settings.
+        
+        Returns:
+            List[Tool]: The list of configured tools
+        """
         try:
-            base_tools = (
-                process_tools(self.config.tools)
-                if self.config.tools is not None
-                else get_default_tools(self.model, enabled_toolboxes=self.config.enabled_toolboxes)
-            )
-            if not self.config.tools_config:
-                return base_tools
+            # Log what enabled_toolboxes is before we use it
+            logger.debug(f"Initializing agent tools with enabled_toolboxes: {self.config.enabled_toolboxes}")
             
-            self._resolve_secrets(self.config.tools_config)
+            # Ensure enabled_toolboxes is a list (fallback to empty list if None)
+            if self.config.enabled_toolboxes is None:
+                self.config.enabled_toolboxes = []
+                logger.warning("Enabled toolboxes was None, defaulting to empty list")
+            
+            # Load default tools from configured toolboxes
+            base_tools = get_default_tools(
+                self.model,
+                history_store=self.conversation_manager.messages if hasattr(self, 'conversation_manager') else None,
+                enabled_toolboxes=self.config.enabled_toolboxes
+            )
+            # Gather per-tool configs from enabled toolboxes
+            tool_confs = []
+            for tb in self.config.installed_toolboxes or []:
+                if tb.enabled:
+                    tool_confs.extend(tb.tool_configs or [])
+            # If no overrides, return base
+            if not tool_confs:
+                return base_tools
+            # Resolve env vars in each config dict
+            for tc in tool_confs:
+                for k, v in getattr(tc, 'config', {}).items():
+                    if isinstance(v, str) and '{{ env.' in v:
+                        env_key = v.strip('{} ').split('.')[-1]
+                        setattr(tc, k, os.getenv(env_key, v))
+            # Apply overrides
             filtered_tools = []
-            processed_names = set()
-            for tool_conf in self.config.tools_config:
-                tool_name = tool_conf.get("name")
-                if tool_conf.get("enabled", True):
-                    tool = next((t for t in base_tools if t.name == tool_name or t.toolbox_name == tool_name), None)
-                    if tool and tool.name not in processed_names:
-                        for key, value in tool_conf.items():
-                            if key not in ["name", "enabled"]:
-                                setattr(tool, key, value)
+            processed = set()
+            for tc in tool_confs:
+                if tc.enabled:
+                    tool = next((t for t in base_tools if t.name == tc.name or t.toolbox_name == tc.name), None)
+                    if tool and tool.name not in processed:
+                        for key, value in tc.config.items():
+                            # Apply the configuration value to the tool
+                            setattr(tool, key, value)
+                            
+                        # Apply confirmation settings if specified in the configuration
+                        if hasattr(tc, 'requires_confirmation'):
+                            tool.requires_confirmation = tc.requires_confirmation
+                            
+                        if hasattr(tc, 'confirmation_message'):
+                            tool.confirmation_message = tc.confirmation_message
                         filtered_tools.append(tool)
-                        processed_names.add(tool.name)
+                        processed.add(tool.name)
+            # Append remaining
             for tool in base_tools:
-                if tool.name not in processed_names:
+                if tool.name not in processed:
                     filtered_tools.append(tool)
             logger.info(f"Loaded {len(filtered_tools)} tools successfully.")
             return filtered_tools
@@ -123,68 +205,53 @@ class Agent:
             logger.error(f"Error resolving secrets in tools_config: {e}")
 
     def _build_system_prompt(self) -> str:
-        """Build a system prompt based on name, personality, backstory, and SOP."""
+        """Render system prompt via Jinja2 template only."""
+        template_name = "system_prompt.j2"
         try:
-            prompt = f"I am {self.name}, an AI assistant." if self.name else "You are an AI assistant."
-            if self.personality:
-                if isinstance(self.personality, str):
-                    prompt += f" I have a {self.personality} personality."
-                elif isinstance(self.personality, dict):
-                    traits = self.personality.get("traits", [])
-                    if traits:
-                        prompt += f" I have the following personality traits: {', '.join(traits)}."
-                    tone = self.personality.get("tone")
-                    if tone:
-                        prompt += f" My tone is {tone}."
-                    humor = self.personality.get("humor_level")
-                    if humor:
-                        prompt += f" My humor level is {humor}."
-            if self.backstory:
-                if isinstance(self.backstory, str):
-                    prompt += f" My backstory is: {self.backstory}"
-                elif isinstance(self.backstory, dict):
-                    origin = self.backstory.get("origin")
-                    if origin:
-                        prompt += f" I was created by {origin}."
-                    purpose = self.backstory.get("purpose")
-                    if purpose:
-                        prompt += f" My purpose is {purpose}."
-                    experience = self.backstory.get("experience")
-                    if experience:
-                        prompt += f" My experience includes: {experience}"
-            if self.sop:
-                prompt += f" Follow this standard operating procedure: {self.sop}"
-            return prompt
+            tpl = self.jinja_env.get_template(template_name)
+            context = {
+                'name': self.name,
+                'personality': self.personality
+            }
+            return tpl.render(**context)
+        except TemplateNotFound:
+            logger.error(f"System prompt template '{template_name}' not found.")
+            return ""
         except Exception as e:
-            logger.error(f"Error building system prompt: {e}. Using default.")
-            return "You are an AI assistant."
+            logger.error(f"Error rendering system prompt: {e}")
+            return ""
 
     async def chat(
         self,
         message: str,
-        history: Optional[List[Dict[str, str]]] = None,
+        history: Optional[List[Message]] = None,
         use_tools: bool = False,
         timeout: int = 30,
         max_tokens: int = MAX_TOKENS,
         temperature: float = None,  # Allow override
         streaming: bool = False,
         reasoner_name: Optional[str] = None,
-        executor_name: Optional[str] = None
+        executor_name: Optional[str] = None,
+        task_id: Optional[str] = None
     ) -> str:
         """Single-step interaction with optional custom tools, history, and streaming."""
         try:
             logger.debug(f"Agent.chat called with message={message!r}, use_tools={use_tools}, timeout={timeout}, max_tokens={max_tokens}, temperature={temperature}, streaming={streaming}, reasoner_name={reasoner_name}, executor_name={executor_name}")
             logger.info(f"Invoking react_agent.chat for simple chat with message={message!r}, streaming={streaming}")
+            # Override conversation history if provided
+            if history is not None:
+                self.conversation_manager.messages = history.copy()
             response: str = await self.react_agent.chat(
                 message,
                 max_tokens=max_tokens,
                 temperature=temperature,
-                streaming=streaming
+                streaming=streaming,
+                task_id=task_id
             )
             logger.info(f"Chat response (no tools): {response}")
             # Update conversation history
-            self.conversation_history_manager.add_message("user", message)
-            self.conversation_history_manager.add_message("assistant", response)
+            self.conversation_manager.add_message("user", message)
+            self.conversation_manager.add_message("assistant", response)
             return response
         except Exception as e:
             logger.exception("Exception in Agent.chat")
@@ -202,7 +269,7 @@ class Agent:
     async def solve(
         self,
         task: str,
-        history: Optional[List[Dict[str, str]]] = None,
+        history: Optional[List[Message]] = None,
         success_criteria: Optional[str] = None,
         task_goal: Optional[str] = None,
         max_iterations: Optional[int] = None,
@@ -210,7 +277,8 @@ class Agent:
         timeout: int = 300,
         streaming: bool = False,
         reasoner_name: Optional[str] = None,
-        executor_name: Optional[str] = None
+        executor_name: Optional[str] = None,
+        task_id: Optional[str] = None
     ) -> List[Dict]:
         """Multi-step task solving with optional custom tools, history, max_iterations, and streaming."""
         try:
@@ -220,16 +288,9 @@ class Agent:
             executor_name = executor_name or self.default_executor_name
             reasoner_cls = self.plugin_manager.reasoners.get(reasoner_name, Reasoner)
             executor_cls = self.plugin_manager.executors.get(executor_name, Executor)
-            reasoner_config = self.config.reasoner.get("config", {})
-            executor_config = self.config.executor.get("config", {})
-            # Format history as a string for the system prompt
-            history_str = ""
-            if history:
-                history_str = "\n".join(
-                    f"{msg['role'].capitalize()}: {msg['content']}" 
-                    for msg in history
-                )
-                system_prompt += f"\n\nPrevious conversation:\n{history_str}"
+            # Unpack config dicts from ReasonerConfig and ExecutorConfig
+            reasoner_config = self.config.reasoner.config
+            executor_config = self.config.executor.config
             solve_agent = CodeActAgent(
                 model=self.model,
                 tools=solve_tools,
@@ -237,27 +298,29 @@ class Agent:
                 max_history_tokens=self.max_history_tokens,
                 system_prompt=system_prompt,
                 reasoner=reasoner_cls(self.model, solve_tools, temperature=self.temperature, **reasoner_config),
-                executor=executor_cls(solve_tools, self._notify_observers, **executor_config),
-                conversation_history_manager=self.conversation_history_manager,
-                temperature=self.temperature
+                executor=executor_cls(solve_tools, self._notify_observers, self.conversation_manager, agent_id=self.id, agent_name=self.name, **executor_config),
+                conversation_manager=self.conversation_manager,
+                temperature=self.temperature,
+                agent_id=self.id,  # New: Pass agent ID
+                agent_name=self.name  # New: Pass agent name
             )
-            solve_agent.executor.register_tool(RetrieveStepTool(solve_agent.history_manager.store))
+            solve_agent.executor.register_tool(RetrieveMessageTool(conversation_manager=self.conversation_manager))
             for observer, event_types in self._observers:
                 solve_agent.add_observer(observer, event_types)
             
+            # Override conversation history if provided
+            if history is not None:
+                self.conversation_manager.messages = history.copy()
             history_result: List[Dict] = await solve_agent.solve(
                 task,
                 success_criteria,
                 task_goal=task_goal,
                 system_prompt=system_prompt,
                 max_iterations=max_iterations,
-                streaming=streaming
+                streaming=streaming,
+                task_id=task_id
             )
             self.last_solve_context_vars = solve_agent.context_vars.copy()
-            # Update conversation history
-            self.conversation_history_manager.add_message("user", f"Task: {task}")
-            final_answer = history_result[-1].get("result", "")
-            self.conversation_history_manager.add_message("assistant", final_answer)
             return history_result
         except Exception as e:
             logger.error(f"Solve failed: {e}")
