@@ -3,5 +3,5 @@ from typing import List
 
 async def clear_command(shell, args: List[str]) -> str:
     """Handle the /clear command."""
-    shell.history_manager.clear_history()
+    shell.conversation_manager.clear_history()
     return "Conversation history cleared."
\ No newline at end of file
