@@ -1,4 +1,3 @@
-from dataclasses import replace
 from typing import List
 
 from ...codeact.agent import Agent
@@ -6,7 +5,7 @@ from ..agent_state import AgentState
 
 
 async def setmodel_command(shell, args: List[str]) -> str:
-    """Set the model by creating and switching to a new agent: /setmodel <model_name>
+    """Set the model by updating the agent config and switching to a new agent: /setmodel <model_name>
     
     Args:
         shell: The Shell instance.
@@ -18,10 +17,10 @@ async def setmodel_command(shell, args: List[str]) -> str:
     if not args:
         return "Please provide a model name (e.g., 'deepseek/deepseek-chat')."
     new_model = args[0]
-    # Create a new config based on the current agent's config, updating only the model
-    new_config = replace(shell.current_agent.config, model=new_model)
+    # Update the centralized agent_config
+    shell.agent_config.model = new_model
     # Instantiate a new agent with the updated config
-    new_agent = Agent(config=new_config)
+    new_agent = Agent(config=shell.agent_config)
     # Add the stream token observer to maintain streaming consistency
     new_agent.add_observer(shell._stream_token_observer, ["StreamToken"])
     # Generate a unique agent name based on the model
