@@ -1,5 +1,5 @@
 import sys
-from difflib import get_close_matches  # Added for command suggestions
+from difflib import get_close_matches
 from importlib.metadata import entry_points
 from pathlib import Path
 from typing import Dict, List, Optional
@@ -15,15 +15,22 @@ from rich.console import Console
 from rich.markdown import Markdown
 from rich.panel import Panel
 
-import quantalogic_codeact.codeact.cli_commands.config_manager as config_manager
-from quantalogic_codeact.codeact.agent import Agent, AgentConfig
-from quantalogic_codeact.codeact.commands.toolbox.get_tool_doc import get_tool_doc
-from quantalogic_codeact.codeact.commands.toolbox.install_toolbox import install_toolbox
-from quantalogic_codeact.codeact.commands.toolbox.list_toolbox_tools import list_toolbox_tools
-from quantalogic_codeact.codeact.commands.toolbox.uninstall_toolbox import uninstall_toolbox
+import quantalogic_codeact.cli_commands.config_manager as config_manager
+from quantalogic_codeact.codeact.agent import Agent
+from quantalogic_codeact.codeact.agent_config import GLOBAL_CONFIG_PATH, GLOBAL_DEFAULTS, AgentConfig
+from quantalogic_codeact.codeact.conversation_manager import ConversationManager
+from quantalogic_codeact.codeact.events import ToolConfirmationRequestEvent
+from quantalogic_codeact.commands.toolbox.disable_toolbox import disable_toolbox
+from quantalogic_codeact.commands.toolbox.enable_toolbox import enable_toolbox
+from quantalogic_codeact.commands.toolbox.get_tool_doc import get_tool_doc
+from quantalogic_codeact.commands.toolbox.install_toolbox import install_toolbox
+from quantalogic_codeact.commands.toolbox.installed_toolbox import installed_toolbox
+from quantalogic_codeact.commands.toolbox.list_toolbox_tools import list_toolbox_tools
+from quantalogic_codeact.commands.toolbox.uninstall_toolbox import uninstall_toolbox
 from quantalogic_codeact.version import get_version
 
 from .agent_state import AgentState
+from .banner import get_welcome_message, print_welcome_banner
 from .command_registry import CommandRegistry
 from .commands.agent import agent_command
 from .commands.chat import chat_command
@@ -33,7 +40,6 @@ from .commands.config_load import config_load
 from .commands.config_save import config_save
 from .commands.config_show import config_show
 from .commands.contrast import contrast_command
-from .commands.debug import debug_command
 from .commands.edit import edit_command
 from .commands.exit import exit_command
 from .commands.help import help_command
@@ -45,14 +51,12 @@ from .commands.loglevel import loglevel_command
 from .commands.mode import mode_command
 from .commands.save import save_command
 from .commands.set import set_command
-from .commands.set_temperature import set_temperature_command  # Added import
+from .commands.set_temperature import set_temperature_command
 from .commands.setmodel import setmodel_command
 from .commands.solve import solve_command
 from .commands.stream import stream_command
 from .commands.tutorial import tutorial_command
 from .commands.version import version_command
-from .history_manager import HistoryManager
-from .shell_state import ShellState
 
 console = Console()
 
@@ -60,11 +64,23 @@ class Shell:
     """Interactive CLI shell for dialog with Quantalogic agents."""
     def __init__(self, agent_config: Optional[AgentConfig] = None, cli_log_level: Optional[str] = None):
         # Configure logger based on config file (default ERROR)
-        cfg = config_manager.load_global_config()
+        self.agent_config = agent_config or AgentConfig.load_from_file(GLOBAL_CONFIG_PATH)
+        # Ensure config is properly initialized with required fields
+        if hasattr(config_manager, 'ensure_config_initialized'):
+            self.agent_config = config_manager.ensure_config_initialized(self.agent_config)
+            
+        # Create default config file if it doesn't exist
+        config_path = Path(GLOBAL_CONFIG_PATH)
+        if not config_path.exists():
+            try:
+                logger.info(f"Creating default configuration file at {config_path}")
+                self.agent_config.save_to_file(str(config_path))
+            except Exception as e:
+                logger.error(f"Failed to create default config file: {e}")
         if cli_log_level:
             level = cli_log_level.upper()
         else:
-            level = cfg.get("log_level", config_manager.GLOBAL_DEFAULTS.get("log_level", "ERROR")).upper()
+            level = self.agent_config.log_level.upper() if hasattr(self.agent_config, 'log_level') else GLOBAL_DEFAULTS.get("log_level", "ERROR").upper()
         logger.remove()
         self.logger_sink_id = logger.add(
             sys.stderr,
@@ -77,56 +93,17 @@ class Shell:
         self.high_contrast = False  # For accessibility
         self.multiline = False  # Default to single-line input
         
-        # Initialize shell state
-        self.state = ShellState(
-            model_name="deepseek/deepseek-chat",
-            max_iterations=10,
-            streaming=True,
-            mode="codeact"
-        )
+        # Initialize conversation manager
+        self.conversation_manager = ConversationManager()
         
-        # Initialize history manager
-        self.history_manager = HistoryManager()
-        
-        # Load or initialize global config
-        if not config_manager.GLOBAL_CONFIG_PATH.exists():
-            default_data = {
-                "model": "gemini/gemini-2.0-flash",
-                "max_iterations": 5,
-                "enabled_toolboxes": [],
-                "reasoner": {"name": "default"},
-                "executor": {"name": "default"},
-                "personality": None,
-                "backstory": None,
-                "sop": None,
-                "name": None,
-                "tools_config": None,
-                "profile": None,
-                "customizations": None,
-                "agent_tool_model": "gemini/gemini-2.0-flash",
-                "agent_tool_timeout": 30,
-                "temperature": 0.7,  # Added default temperature
-                "installed_toolboxes": [],
-                "log_level": config_manager.GLOBAL_DEFAULTS.get("log_level", "ERROR")
-            }
-            config_manager.save_global_config(default_data)
-            logger.info(f"Created global configuration at {config_manager.GLOBAL_CONFIG_PATH}")
-            config_data = default_data
-        else:
-            config_data = config_manager.load_global_config()
-        # Prepare AgentConfig args
-        config_data.pop("installed_toolboxes", None)
-        config_data.pop("log_level", None)
-        try:
-            default_config = AgentConfig(**config_data)
-            logger.info(f"Loaded configuration from {config_manager.GLOBAL_CONFIG_PATH}")
-        except Exception as e:
-            logger.error(f"Failed to initialize AgentConfig: {e}. Using minimal configuration.")
-            default_config = AgentConfig()
-        
-        # Initialize the default agent with the loaded or default config
-        default_agent = Agent(config=agent_config or default_config)
+        # Debug log for enabled_toolboxes value before agent initialization
+        logger.debug(f"Shell initializing agent with enabled_toolboxes: {self.agent_config.enabled_toolboxes}")
+    
+        # Initialize the default agent with the loaded config
+        default_agent = Agent(config=self.agent_config)
         default_agent.add_observer(self._stream_token_observer, ["StreamToken"])
+        # Subscribe to tool confirmation events - this is critical for proper confirmation handling
+        default_agent.add_observer(self._handle_tool_confirmation, ["ToolConfirmationRequest"])
         self.agents: Dict[str, AgentState] = {
             "default": AgentState(agent=default_agent)
         }
@@ -145,7 +122,11 @@ class Shell:
     @property
     def current_message_history(self) -> List[Dict[str, str]]:
         """Get the current agent's message history."""
-        return self.history_manager.get_history()
+        # Convert Message objects to dicts for compatibility with history_command
+        return [
+            {"role": msg.role, "content": msg.content, "nanoid": msg.nanoid}
+            for msg in self.conversation_manager.get_history()
+        ]
 
     def _register_builtin_commands(self) -> None:
         """Register all built-in commands with their arguments for autocompletion."""
@@ -154,14 +135,13 @@ class Shell:
             {"name": "chat", "func": chat_command, "help": "Chat with the agent: /chat <message>", "args": None},
             {"name": "solve", "func": solve_command, "help": "Solve a task: /solve <task>", "args": None},
             {"name": "compose", "func": compose_command, "help": "Compose input in external editor: /compose", "args": []},
-            {"name": "edit", "func": edit_command, "help": "Edit last user message: /edit", "args": []},
+            {"name": "edit", "func": edit_command, "help": "Edit a previous user message: /edit [INDEX_OR_ID]", "args": []},
             {"name": "exit", "func": exit_command, "help": "Exit the shell: /exit", "args": []},
             {"name": "history", "func": history_command, "help": "Show conversation history: /history [n]", "args": None},
             {"name": "clear", "func": clear_command, "help": "Clear conversation history: /clear", "args": []},
             {"name": "stream", "func": stream_command, "help": "Toggle streaming: /stream on|off", "args": ["on", "off"]},
             {"name": "mode", "func": mode_command, "help": "Set or show mode: /mode [chat|codeact]", "args": ["chat", "codeact"]},
             {"name": "loglevel", "func": loglevel_command, "help": "Set log level: /loglevel [DEBUG|INFO|WARNING|ERROR|CRITICAL]", "args": ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]},
-            {"name": "debug", "func": debug_command, "help": "Toggle debug mode: /debug on|off", "args": ["on", "off"]},
             {"name": "save", "func": save_command, "help": "Save history: /save <filename>", "args": None},
             {"name": "load", "func": load_command, "help": "Load history: /load <filename>", "args": None},
             {"name": "agent", "func": agent_command, "help": "Switch agent: /agent <name>", "args": list(self.agents.keys())},
@@ -171,26 +151,51 @@ class Shell:
             {"name": "setmodel", "func": setmodel_command, "help": "Set model and switch to a new agent: /setmodel <model_name>", "args": None},
             {"name": "set", "func": set_command, "help": "Set a config field: /set <field> <value>", "args": None},
             {"name": "set temperature", "func": set_temperature_command, "help": "Set or show temperature: /set temperature <value>", "args": None},
+            {"name": "config", "func": config_show, "help": "Show the current configuration", "args": []},
             {"name": "config show", "func": config_show, "help": "Show the current configuration", "args": []},
             {"name": "config save", "func": config_save, "help": "Save the current configuration to a file: /config save <filename>", "args": None},
             {"name": "config load", "func": config_load, "help": "Load a configuration from a file: /config load <filename>", "args": None},
+            {"name": "toolbox", "func": list_toolbox_tools, "help": "List tools in enabled toolboxes: /toolbox [<toolbox_name>]", "args": None},
             {"name": "toolbox install", "func": install_toolbox, "help": "Install a toolbox: /toolbox install <toolbox_name>", "args": None},
             {"name": "toolbox uninstall", "func": uninstall_toolbox, "help": "Uninstall a toolbox: /toolbox uninstall <toolbox_name>", "args": None},
+            {"name": "toolbox installed", "func": installed_toolbox, "help": "Show installed toolboxes: /toolbox installed", "args": None},
+            {"name": "toolbox enable", "func": enable_toolbox, "help": "Enable a toolbox: /toolbox enable <toolbox_name>", "args": None},
+            {"name": "toolbox disable", "func": disable_toolbox, "help": "Disable a toolbox: /toolbox disable <toolbox_name>", "args": None},
             {"name": "toolbox tools", "func": list_toolbox_tools, "help": "List tools in a toolbox: /toolbox tools <toolbox_name>", "args": None},
             {"name": "toolbox doc", "func": get_tool_doc, "help": "Show tool documentation: /toolbox doc <toolbox_name> <tool_name>", "args": None},
             {"name": "listmodels", "func": listmodels_command, "help": "List models using LLM util: /listmodels", "args": []},
             {"name": "version", "func": version_command, "help": "Show package version: /version", "args": []},
+            {"name": "banner", "func": self.banner_command, "help": "Display the welcome banner again", "args": []},
         ]
         for cmd in builtin_commands:
-            self.command_registry.register(
-                cmd["name"],
-                lambda args, f=cmd["func"]: f(self, args),
-                cmd["help"],
-                cmd["args"]
-            )
+            if cmd["name"] == "banner":
+                # Directly use the bound method for banner
+                self.command_registry.register(
+                    cmd["name"],
+                    lambda args=None: cmd["func"](args or []),
+                    cmd["help"],
+                    cmd["args"]
+                )
+            else:
+                # Standard registration for other commands
+                self.command_registry.register(
+                    cmd["name"],
+                    lambda args, f=cmd["func"], s=self: f(s, args),
+                    cmd["help"],
+                    cmd["args"]
+                )
         # Set args for /help to include all command names for autocompletion
         self.command_registry.commands["help"]["args"] = list(self.command_registry.commands.keys())
 
+    async def banner_command(self, args: List[str]) -> None:
+        """Display the welcome banner with consistent colors."""
+        agent_name = getattr(self.agent_config, 'agent_name', 'QUANTA') or 'QUANTA'
+        print_welcome_banner(
+            agent_name,
+            self.agent_config.mode,
+            self.high_contrast
+        )
+
     def _load_plugin_commands(self) -> None:
         """Load plugin commands from entry points."""
         try:
@@ -207,6 +212,75 @@ class Shell:
     async def _stream_token_observer(self, event: object) -> None:
         """Observer for streaming tokens (handled in commands)."""
         pass
+        
+    async def _handle_tool_confirmation(self, event: ToolConfirmationRequestEvent) -> None:
+        """Handle tool confirmation requests by interacting with the user.
+        
+        This method is called when a tool requires confirmation before execution.
+        It displays a prompt to the user and sets the confirmation result on the event's future.
+        """
+        if event.event_type != "ToolConfirmationRequest":
+            return
+        
+        # Add debug logging    
+        logger.debug(f"Received tool confirmation request for: {event.tool_name}")
+        logger.debug(f"Confirmation message: {event.confirmation_message}")
+            
+        # Format parameter display nicely
+        formatted_params = '\n'.join(f"  - {k}: {v}" for k, v in event.parameters_summary.items())
+        
+        # Display confirmation panel with message and parameters
+        panel = Panel(
+            f"Tool: {event.tool_name}\n\nParameters:\n{formatted_params}\n\n{event.confirmation_message}",
+            title="Confirmation Required", 
+            border_style="yellow"
+        )
+        console.print(panel)
+        console.print("[bold yellow]Type [bold white]'yes'[/bold white] or [bold white]'no'[/bold white] to confirm or cancel this operation.[/bold yellow]")
+        
+        # Log that we're waiting for confirmation
+        logger.debug("Shell is now getting confirmation input directly")
+        
+        # DIRECT HANDLING: Get and process input directly
+        try:
+            # Use direct Python input() instead of prompt_toolkit
+            response = input("Confirm (yes/no): ").strip().lower()
+            console.print(f"[italic]Received response: '{response}'[/italic]")
+            
+            # Process the response directly
+            if response not in ['yes', 'no']:
+                console.print("[bold red]Invalid response. Please enter exactly 'yes' or 'no'.[/bold red]")
+                # Try again once more
+                response = input("Confirm (yes/no): ").strip().lower()
+                if response not in ['yes', 'no']:
+                    console.print("[bold red]Invalid response again. Treating as 'no'.[/bold red]")
+                    response = 'no'
+            
+            # Process confirmation
+            confirmed = (response == 'yes')
+            logger.debug(f"Confirmation response: {confirmed}")
+            
+            # Set the result on the event's future
+            if event.confirmation_future and not event.confirmation_future.done():
+                event.confirmation_future.set_result(confirmed)
+                logger.debug("Successfully set confirmation result on future")
+                
+                # Show result to user with clearer feedback
+                if confirmed:
+                    console.print("[bold green]Confirmation accepted - proceeding with operation[/bold green]")
+                else:
+                    console.print("[bold red]Confirmation declined - ENTIRE TASK ABORTED[/bold red]")
+                    console.print("[italic]The entire task execution has been stopped because you answered 'no'[/italic]")
+                    console.print("[dim]This is the behavior you requested - to stop the solve task on confirmation decline[/dim]")
+            else:
+                logger.error("No valid confirmation future found in event or future already done")
+                console.print("[bold orange3]Confirmation response not processed - no valid response mechanism found.[/bold orange3]")
+        except Exception as e:
+            logger.error(f"Error getting confirmation input: {e}")
+            console.print(f"[bold red]Error getting confirmation input: {e}[/bold red]")
+            # Set the future to False if it exists and is not done
+            if event.confirmation_future and not event.confirmation_future.done():
+                event.confirmation_future.set_result(False)
 
     def bottom_toolbar(self):
         """Render a bottom toolbar with mode, agent, model, temperature, and version information as prompt_toolkit HTML."""
@@ -217,9 +291,10 @@ class Shell:
             # High-contrast mode: single style with all info
             return HTML(
                 f'<b><style fg="ansiblack" bg="#E6E6FA">'
-                f'Mode: {self.state.mode} | '
+                f'Mode: {self.agent_config.mode} | '
                 f'Agent: {self.current_agent.name or "Default"} | '
                 f'Model: {self.current_agent.model} | '
+                f'MaxIter: {self.agent_config.max_iterations} | '
                 f'Temperature: {temperature:.2f} | '
                 f'Version: {get_version()}'
                 f'</style></b>'
@@ -228,9 +303,10 @@ class Shell:
             # Default mode: multi-colored sections
             return HTML(
                 f'<b>'
-                f'<style fg="ansiwhite" bg="ansired"> Mode: {self.state.mode} </style>'
+                f'<style fg="ansiwhite" bg="ansired"> Mode: {self.agent_config.mode} </style>'
                 f'<style fg="ansiwhite" bg="ansigreen"> Agent: {self.current_agent.name or "Default"} </style>'
                 f'<style fg="ansiwhite" bg="ansiblue"> Model: {self.current_agent.model} </style>'
+                f'<style fg="ansiwhite" bg="ansicyan"> MaxIter: {self.agent_config.max_iterations} </style>'
                 f'<style fg="ansiwhite" bg="ansiyellow"> Temperature: {temperature:.2f} </style>'
                 f'<style fg="ansiwhite" bg="ansimagenta"> Version: {get_version()} </style>'
                 f'</b>'
@@ -260,9 +336,9 @@ class Shell:
         })
         session = PromptSession(
             message=lambda: HTML(
-                f'<ansigreen>[cfg:{config_manager.GLOBAL_CONFIG_PATH.name}]</ansigreen> '
+                f'<ansigreen>[cfg:{GLOBAL_CONFIG_PATH.name}]</ansigreen> '
                 f'<ansicyan>[{self.current_agent.name or "Agent"}]</ansicyan> '
-                f'<ansiyellow>[{self.state.mode}]></ansiyellow> '
+                f'<ansiyellow>[{self.agent_config.mode}]></ansiyellow> '
             ),
             completer=completer,
             multiline=self.multiline,
@@ -273,13 +349,7 @@ class Shell:
         self.session = session  # Store for inputmode updates
 
         # Welcome message
-        welcome_message = (
-            f"Welcome to Quantalogic Shell (v{get_version()}).\n\n"
-            f"Interacting with agent: {self.current_agent.name or 'Agent'}\n"
-            f"Mode: {self.state.mode} - plain messages are "
-            f"{'tasks to solve' if self.state.mode == 'codeact' else 'chat messages'}.\n\n"
-            f"Type /help for commands. Press Enter to send, Ctrl+J for new lines."
-        )
+        welcome_message = get_welcome_message(self.current_agent.name, self.agent_config.mode)
         console.print(Panel(welcome_message, title="Quantalogic Shell", border_style="blue"))
 
         while True:
@@ -294,7 +364,7 @@ class Shell:
                 user_input = user_input.strip()
                 if not user_input:
                     continue
-
+                
                 # Display user input with label and color
                 color = "bright_blue" if self.high_contrast else "blue"
                 console.print(f"[bold {color}]User:[/bold {color}] {user_input}")
@@ -319,7 +389,7 @@ class Shell:
                         if result and matched_command not in ["chat", "solve", "tutorial"]:
                             # Use Markdown rendering for toolbox commands
                             border_color = "bright_blue" if self.high_contrast else "blue"
-                            if matched_command in ["toolbox doc", "toolbox tools"]:
+                            if matched_command in ["toolbox doc", "toolbox tools", "toolbox"]:
                                 console.print(Markdown(result))
                             else:
                                 title = "Conversation History" if matched_command == "history" else f"{matched_command.capitalize()} Command"
@@ -335,7 +405,7 @@ class Shell:
                         console.print(Panel(error_message, title="Error", border_style=border_style))
                 else:
                     # Handle non-command input based on mode
-                    if self.state.mode == "codeact":
+                    if self.agent_config.mode == "codeact":
                         await solve_command(self, [user_input])
                     else:  # chat mode
                         await chat_command(self, [user_input])
